
# q-expansion of newform 6336.2.b.i, downloaded from the LMFDB on 20 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6336
weight = 2
poly_data = [2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[4159, [1, 0]], [4357, [1, 0]], [3521, [-1, 0]], [1729, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 2], [0, 0], [3, -1], [0, -3], [-6, 0], [0, 3], [0, -1], [0, 0], [-2, 0], [10, 0], [-6, 0], [0, -9], [0, -7], [0, -4], [0, 4], [0, -9], [8, 0], [0, 11], [0, -6], [0, 6], [6, 0], [0, -5], [-4, 0], [6, 0], [-14, 0], [-12, 0], [0, 3], [0, -5], [0, -6], [12, 0], [0, 1], [0, -9], [6, 0], [0, -6], [10, 0], [-16, 0], [0, 0], [12, 0], [0, 10], [-14, 0], [0, 5], [0, -6], [18, 0], [16, 0], [0, 15], [-8, 0], [-12, 0], [22, 0], [18, 0], [-12, 0], [0, -12], [0, -14], [0, 1], [0, 0], [0, -4], [0, -6], [0, 3], [30, 0], [0, -9], [0, 0], [0, 15], [0, -1], [8, 0], [0, -16], [20, 0], [0, -6], [30, 0], [0, 21], [0, 7], [12, 0], [16, 0], [0, -15], [20, 0], [0, -7], [0, -10], [-2, 0], [0, 1], [0, -24], [0, 10], [-26, 0], [-12, 0], [-16, 0], [0, -24], [0, 28], [0, 7], [0, -12], [-12, 0], [22, 0], [0, 10], [-36, 0], [-8, 0], [-12, 0], [-4, 0], [-12, 0], [0, 8], [0, 25], [0, -9], [0, -15], [0, 9], [12, 0], [6, 0], [-6, 0], [0, -3], [-4, 0], [0, -20], [-30, 0], [0, 17], [0, 0], [0, 6], [0, -9], [0, -17], [-40, 0], [22, 0], [0, -11], [32, 0], [0, 23], [0, -22], [-18, 0], [-2, 0], [0, 30], [6, 0], [0, 22], [20, 0], [-48, 0], [34, 0], [0, -1], [40, 0], [0, 3], [0, -15], [-36, 0], [-2, 0], [34, 0], [-42, 0], [0, -6], [0, -4], [0, 15], [0, -22], [-18, 0], [0, -33], [18, 0], [-32, 0], [42, 0], [-14, 0], [0, -7], [0, -3], [30, 0], [8, 0], [0, 5], [0, -27], [0, -17], [-16, 0], [-36, 0], [8, 0], [0, 11], [0, -18], [0, 7], [0, -24], [-30, 0], [0, -32], [6, 0], [0, 18], [0, 4], [0, -17], [0, 35], [34, 0], [0, 21], [0, 36], [0, -10], [36, 0], [-14, 0], [36, 0], [0, 6], [-8, 0], [0, -11], [0, 21], [0, -22], [0, 24], [0, 9], [-62, 0], [-60, 0], [-14, 0], [-42, 0], [0, 11], [0, 2], [0, 3], [44, 0], [0, 12], [60, 0], [-34, 0], [6, 0], [-64, 0], [0, -34], [-54, 0], [0, 7], [0, 36], [-14, 0], [42, 0], [-36, 0], [30, 0], [0, 12], [-14, 0], [0, 6], [0, -2], [0, 20], [-38, 0], [-12, 0], [-6, 0], [-4, 0], [0, -6], [0, -46], [-2, 0], [0, -2], [24, 0], [38, 0], [0, 24], [54, 0], [0, 41], [0, -22], [0, 33], [0, -18], [0, -29], [-32, 0], [-18, 0], [0, 33], [0, -29], [0, 23], [0, -24], [36, 0], [46, 0], [0, 33], [0, 36], [-18, 0], [44, 0], [-36, 0], [-52, 0], [-12, 0], [0, 46], [0, -43], [0, -14], [0, 33], [-14, 0], [-62, 0], [-30, 0], [24, 0], [16, 0], [0, -44], [0, -33], [12, 0], [0, 45], [-42, 0], [0, -43], [-64, 0], [-18, 0], [-30, 0], [34, 0], [0, 3], [0, -4], [0, -42], [0, 54], [36, 0], [0, 27], [0, -33], [0, 37], [-28, 0], [0, -58], [0, -23], [0, -15], [-36, 0], [10, 0], [-40, 0], [26, 0], [0, -48], [0, -30], [16, 0], [0, -8], [0, -27], [0, -36], [6, 0], [-24, 0], [-80, 0], [48, 0], [0, -21], [0, -3], [0, 17], [-52, 0], [-36, 0], [40, 0], [-30, 0], [0, 2], [0, -32], [-42, 0], [-6, 0], [0, -21], [-66, 0], [40, 0], [0, 56], [-54, 0], [0, -3], [0, 54], [-12, 0], [0, -12], [0, -26], [32, 0], [-34, 0], [0, 40], [-50, 0], [0, -19], [0, -39], [60, 0], [0, -4], [66, 0], [20, 0], [-60, 0], [0, 42], [0, 22], [12, 0], [8, 0], [-30, 0], [0, -51], [32, 0], [42, 0], [64, 0], [6, 0], [38, 0], [32, 0], [-28, 0], [-72, 0], [36, 0], [0, 27], [0, 26], [0, 12], [12, 0], [0, 9], [0, -2], [94, 0], [-6, 0], [80, 0], [0, 6], [46, 0], [0, 67], [24, 0], [-80, 0], [0, 20], [18, 0], [46, 0], [-28, 0], [-72, 0], [0, -16], [0, 33], [-4, 0], [0, 8], [0, -30], [0, 3], [-42, 0], [0, -49], [-60, 0], [-18, 0], [0, 23], [0, -3], [42, 0], [0, -49], [60, 0], [20, 0], [14, 0], [18, 0], [0, 30], [0, 30], [0, -44], [-16, 0], [12, 0], [54, 0], [0, -36], [34, 0], [0, -8], [-24, 0], [0, 0], [-90, 0], [0, 0], [0, 2], [0, 1], [0, 42], [42, 0], [0, 33], [0, -13], [-26, 0], [-74, 0], [0, -3], [0, 65], [-16, 0], [0, 62], [0, -26], [44, 0], [0, 11], [0, 48], [0, 0], [0, 43], [68, 0], [60, 0], [0, -3], [0, 55], [0, 12], [0, 37], [-24, 0], [0, -12], [46, 0], [78, 0], [-4, 0], [0, 16], [0, 60], [42, 0], [0, -68], [0, 69], [0, -30], [0, 44], [12, 0], [-38, 0], [0, -5], [36, 0], [0, 50], [0, -21], [0, -61], [-12, 0], [-16, 0], [0, 20], [0, 40], [18, 0], [44, 0], [84, 0]]
